/*
-----------------------------------------------------------------------------
This source file is part of OGRE
    (Object-oriented Graphics Rendering Engine)
For the latest info, see http://www.ogre3d.org/

Copyright (c) 2000-2005 The OGRE Team
Also see acknowledgements in Readme.html

You may use this sample code for anything you like, it is not covered by the
LGPL like the rest of the engine.
-----------------------------------------------------------------------------
*/

/**
    \file 
        Lighting.h
    \brief
        Shows lighting support in OGRE. Also demonstrates the use of billboards
        and controllers for automatic time-relative behaviour.
*/


#include "ExampleApplication.h"
#include "OgreIlluminationManager.h"
#include "SceneSerializer.h"


// Listener class for frame updates
class MoriaDemoFrameListener : public ExampleFrameListener
{
protected:
	
	unsigned long framecount;
	SceneNode* mainLightNode;
	Light* light;
	
public:
    MoriaDemoFrameListener(RenderWindow* window, Camera* maincam, SceneNode* lightNode, Light* l)
        :ExampleFrameListener(window, maincam) 
				 
    {
		mainLightNode = lightNode;
		light = l;
		framecount = 0;	
		mMoveSpeed /= 50.0;
    }

    bool frameStarted(const FrameEvent& evt)
    {	
	/*	light->setDirection(mCamera->getDirection());
		light->setPosition(mCamera->getPosition());
		mainLightNode->setPosition(mCamera->getPosition());*/
		bool result = ExampleFrameListener::frameStarted(evt);
       framecount++;
	  
        return result;     
    }

	

};

/** Application class */
class MoriaDemoApplication : public ExampleApplication
{




protected:
	SceneNode* kupolaNode;
	SceneNode* mainLightNode;
	Light* mainLight;
	
	void loadResources(void)
	{
		// Initialise, parse scripts etc
		ResourceGroupManager::getSingleton().initialiseResourceGroup("Bootstrap");
		ResourceGroupManager::getSingleton().initialiseResourceGroup("GTP_Basic");
		ResourceGroupManager::getSingleton().initialiseResourceGroup("General");
		ResourceGroupManager::getSingleton().initialiseResourceGroup("PostProc");
		ResourceGroupManager::getSingleton().initialiseResourceGroup("GTP_Particles");
		ResourceGroupManager::getSingleton().initialiseResourceGroup("GTP_Moria");
		ResourceGroupManager::getSingleton().initialiseResourceGroup("GTP_EnvMap");
		ResourceGroupManager::getSingleton().initialiseResourceGroup("GTP_Caustic");
	}

	void createPostproc()
	{		
	/*	CompositorManager::getSingleton().addCompositor(mWindow->getViewport(0),
														"GTP/PostProc/Glow");
		CompositorManager::getSingleton().setCompositorEnabled(mWindow->getViewport(0),
														"GTP/PostProc/Glow", true);

		CompositorManager::getSingleton().addCompositor(mWindow->getViewport(0),
														"GTP/PostProc/ToneMap");
		CompositorManager::getSingleton().setCompositorEnabled(mWindow->getViewport(0),
														"GTP/PostProc/ToneMap", true);	*/

		CompositorManager::getSingleton().addCompositor(mWindow->getViewport(0),
														"GTP/PostProc/Plain");
		CompositorManager::getSingleton().setCompositorEnabled(mWindow->getViewport(0),
														"GTP/PostProc/Plain", true);	
		
	}
    
	void chooseSceneManager()
	{
		//mSceneMgr = mRoot->createSceneManager("OctreeSceneManager");
		mSceneMgr = mRoot->createSceneManager("OcclusionCullingSceneManager");
	}

	void createScene(void) ///TOWERS
    {
		std::string occlusion_cfg("OcclusionCulling.cfg");
		mSceneMgr->setWorldGeometry(occlusion_cfg);
		
		//createPostproc();

		OgreIlluminationManager::getSingleton().setMainCamera(mCamera);
		OgreIlluminationManager::getSingleton().setMainViewport(mWindow->getViewport(0));
		 OgreIlluminationManager::getSingleton().setFocusingSM(false);
	   OgreIlluminationManager::getSingleton().setShadowMapSize(512);
	   OgreIlluminationManager::getSingleton().setBlurShadowMap(true);
	   OgreIlluminationManager::getSingleton().setShadowMapMaterialName("GTP/Basic/Distance_Normalized");//GameTools/ShadowMapDistance
	  OgreIlluminationManager::getSingleton().setUseLISPSM(false);
				
		Root::getSingleton()._setCurrentSceneManager(mSceneMgr);
		mCamera->setPosition(-20,3,0);
		//mCamera->setPosition(0,100,0);
		//mCamera->lookAt(0,-1,0);
		mCamera->setFOVy(Radian(Degree(80)));
		mCamera->setNearClipDistance(0.01);
		mCamera->setFarClipDistance(1000);
		// Set ambient light
        mSceneMgr->setAmbientLight(ColourValue(0.1, 0.1, 0.1));
		mSceneMgr->setSkyBox(true, "Examples/EveningSkyBox", 500);
		
		SceneNode* rootNode = mSceneMgr->getRootSceneNode();

	  
		SceneSerializer s(mSceneMgr);
		DataStreamPtr inputStream; 				
		inputStream = ResourceGroupManager::getSingleton().openResource("moria.level");	//towers2.level	
		s.parseScript(inputStream, ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME);

	   ParticleSystem* pSys1 = mSceneMgr->createParticleSystem("psys1", "GTP/Moria/Smoke_Large");
	   pSys1->setKeepParticlesInLocalSpace(false);
	   SceneNode *pSysNode = rootNode->createChildSceneNode("psys1");
	   pSysNode->attachObject(pSys1);
	   pSysNode->setPosition(-40,-1,-15);
		
	   /*Entity* object = mSceneMgr->createEntity("object", "head.mesh");
		object->setMaterialName("GTP/Caustic/Glass");
		SceneNode* objectNode = rootNode->createChildSceneNode();
		objectNode->attachObject(object);
		objectNode->scale(0.1,0.1,0.1);
		objectNode->rotate(Vector3(0,1,0),Radian(0.2));
		objectNode->setPosition(-30,2.0,-5);
		objectNode->_updateBounds();*/
			
		//createPlane("p","groundmaterial",Vector3(0,-0.3,0),Vector2(150,150));

		Light* mainLight = mSceneMgr->createLight("MainLight");
		mainLight->setType(Light::LT_SPOTLIGHT);
		mainLight->setCastShadows(true);
        mainLight->setDiffuseColour(ColourValue::White);
		mainLight->setSpecularColour(ColourValue::White);
		mainLightNode = rootNode->createChildSceneNode();
		
		mainLightNode->setPosition(-20,10,0);		
		mainLight->setPosition(-20,10,0);
		mainLight->setPowerScale(5);
		mainLight->setDirection(-1,-0.2, -1);
		mainLightNode->attachObject(mainLight);
		mainLight->setSpotlightRange(Degree(0),Degree(90));
		mainLight->setAttenuation(40.0, 0, 1, 0);
		
		OgreIlluminationManager::getSingleton().initTechniques();
   }
	
	void createPlane(Ogre::String _name, Ogre::String _material, Ogre::Vector3 _pos, Ogre::Vector2 _size, Ogre::Vector2 _subdivisions = Ogre::Vector2(1,1)) 
	{
		Plane _plane;
        _plane.normal = Vector3::UNIT_Y;
        _plane.d = 0;


        MeshManager::getSingleton().createPlane(_name + ".plane",
            ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME, 
            _plane,_size.x,_size.y,1,1,true,1,_subdivisions.x,_subdivisions.y,Vector3::UNIT_Z);
        
		Entity *_entity = mSceneMgr->createEntity(_name + ".entity", _name + ".plane");
        _entity->setMaterialName(_material);

		SceneNode* _node = mSceneMgr->getRootSceneNode()->createChildSceneNode(_name + ".node");
		_node->attachObject(_entity);
		_node->setPosition(_pos);

	}

	void createFrameListener(void)
    {
		// This is where we instantiate our own frame listener
		mFrameListener= new MoriaDemoFrameListener(mWindow,
											mCamera, mainLightNode, mainLight	);
       mFrameListener->setPriority(2);	
		mRoot->addFrameListener(mFrameListener);
		OgreIlluminationManager::getSingleton().setPriority(3);
		mRoot->addFrameListener(&OgreIlluminationManager::getSingleton());
    }

};

